pub enum Error {
LastUsedWithoutPreviousValue,
InvalidType,
NotImplemented,
InvalidNumber,
DivideByZero,
UnknownFunction(String),
Function(FunctionError),
}Expand description
An error that occured while evaluating an expression
Variants
LastUsedWithoutPreviousValue
LAST FieldName was used but no previous value was provided
InvalidType
The expression expected a certain type, but the value was not the expected type
NotImplemented
Some aspect of evaluation is not implemented
InvalidNumber
The expression did not result in a number that can be used
DivideByZero
The expression attempted to divide by zero
UnknownFunction(String)
A function call was used to a function that does not exist
Function(FunctionError)
A function call failed