pub enum Expression {
Field(FieldNode),
LastField(LastFieldNode),
And(AndNode),
Or(OrNode),
Not(NotNode),
Op(OpNode),
Literal(LiteralNode),
Function(FunctionNode),
Iif(IifNode),
List(ListNode),
}Expand description
An expression that can be inspected or evaluated
Variants
Field(FieldNode)
A node representing a reference to a field
E.g. MlsStatus
LastField(LastFieldNode)
A node representing a reference to a field from the previous value
E.g. LAST MlsStatus
And(AndNode)
A node representing the logical conjunction of two or more expressions
E.g. A .AND. B
Or(OrNode)
A node representing the logical disjunction of two or more expressions
E.g. A .OR. B
Not(NotNode)
A node representing the logical negation of an expression
E.g. .NOT. A
Op(OpNode)
A node representing a binary operation
E.g. A + B
Literal(LiteralNode)
A node representing a literal value
E.g. .TRUE.
Function(FunctionNode)
A node representing a function call
E.g. SUBSTR('A', 1, 2)
Iif(IifNode)
A node representing the special IIF conditional syntax
E.g. IIF(A, B, C)
List(ListNode)
A node representing a list of items
E.g. (1, 2, 3)
Implementations
Sourceimpl Expression
impl Expression
Sourceimpl Expression
impl Expression
Sourceimpl Expression
impl Expression
Sourcepub fn as_field(&self) -> Option<&FieldNode>
pub fn as_field(&self) -> Option<&FieldNode>
If the Expression is a X, returns a reference to the associated Y. Returns None
otherwise.
Sourcepub fn as_field_mut(&mut self) -> Option<&mut FieldNode>
pub fn as_field_mut(&mut self) -> Option<&mut FieldNode>
If the Expression is a X, returns a mutable reference to the associated Y. Returns
None otherwise.
Sourcepub fn as_last_field(&self) -> Option<&LastFieldNode>
pub fn as_last_field(&self) -> Option<&LastFieldNode>
If the Expression is a X, returns a reference to the associated Y. Returns None
otherwise.
Sourcepub fn as_last_field_mut(&mut self) -> Option<&mut LastFieldNode>
pub fn as_last_field_mut(&mut self) -> Option<&mut LastFieldNode>
If the Expression is a X, returns a mutable reference to the associated Y. Returns
None otherwise.
Sourcepub fn as_and(&self) -> Option<&AndNode>
pub fn as_and(&self) -> Option<&AndNode>
If the Expression is a X, returns a reference to the associated Y. Returns None
otherwise.
Sourcepub fn as_and_mut(&mut self) -> Option<&mut AndNode>
pub fn as_and_mut(&mut self) -> Option<&mut AndNode>
If the Expression is a X, returns a mutable reference to the associated Y. Returns
None otherwise.
Sourcepub fn as_or(&self) -> Option<&OrNode>
pub fn as_or(&self) -> Option<&OrNode>
If the Expression is a X, returns a reference to the associated Y. Returns None
otherwise.
Sourcepub fn as_or_mut(&mut self) -> Option<&mut OrNode>
pub fn as_or_mut(&mut self) -> Option<&mut OrNode>
If the Expression is a X, returns a mutable reference to the associated Y. Returns
None otherwise.
Sourcepub fn as_not(&self) -> Option<&NotNode>
pub fn as_not(&self) -> Option<&NotNode>
If the Expression is a X, returns a reference to the associated Y. Returns None
otherwise.
Sourcepub fn as_not_mut(&mut self) -> Option<&mut NotNode>
pub fn as_not_mut(&mut self) -> Option<&mut NotNode>
If the Expression is a X, returns a mutable reference to the associated Y. Returns
None otherwise.
Sourcepub fn as_op(&self) -> Option<&OpNode>
pub fn as_op(&self) -> Option<&OpNode>
If the Expression is a X, returns a reference to the associated Y. Returns None
otherwise.
Sourcepub fn as_op_mut(&mut self) -> Option<&mut OpNode>
pub fn as_op_mut(&mut self) -> Option<&mut OpNode>
If the Expression is a X, returns a mutable reference to the associated Y. Returns
None otherwise.
Sourcepub fn as_literal(&self) -> Option<&LiteralNode>
pub fn as_literal(&self) -> Option<&LiteralNode>
If the Expression is a X, returns a reference to the associated Y. Returns None
otherwise.
Sourcepub fn as_literal_mut(&mut self) -> Option<&mut LiteralNode>
pub fn as_literal_mut(&mut self) -> Option<&mut LiteralNode>
If the Expression is a X, returns a mutable reference to the associated Y. Returns
None otherwise.
Sourcepub fn as_function(&self) -> Option<&FunctionNode>
pub fn as_function(&self) -> Option<&FunctionNode>
If the Expression is a X, returns a reference to the associated Y. Returns None
otherwise.
Sourcepub fn as_function_mut(&mut self) -> Option<&mut FunctionNode>
pub fn as_function_mut(&mut self) -> Option<&mut FunctionNode>
If the Expression is a X, returns a mutable reference to the associated Y. Returns
None otherwise.
Sourcepub fn as_iif(&self) -> Option<&IifNode>
pub fn as_iif(&self) -> Option<&IifNode>
If the Expression is a X, returns a reference to the associated Y. Returns None
otherwise.
Sourcepub fn as_iif_mut(&mut self) -> Option<&mut IifNode>
pub fn as_iif_mut(&mut self) -> Option<&mut IifNode>
If the Expression is a X, returns a mutable reference to the associated Y. Returns
None otherwise.
Sourcepub fn as_list(&self) -> Option<&ListNode>
pub fn as_list(&self) -> Option<&ListNode>
If the Expression is a X, returns a reference to the associated Y. Returns None
otherwise.
Sourcepub fn as_list_mut(&mut self) -> Option<&mut ListNode>
pub fn as_list_mut(&mut self) -> Option<&mut ListNode>
If the Expression is a X, returns a mutable reference to the associated Y. Returns
None otherwise.
Sourceimpl Expression
impl Expression
Sourcepub fn apply<'expr, 'val, T>(
&'expr self,
context: EvaluateContext<'_, 'val, T>,
) -> Result<Cow<'val, Value>, Error>where
'expr: 'val,
pub fn apply<'expr, 'val, T>(
&'expr self,
context: EvaluateContext<'_, 'val, T>,
) -> Result<Cow<'val, Value>, Error>where
'expr: 'val,
Run the expression to completion and return the result
Sourcepub fn apply_with_locals<'expr, 'val, T>(
&'expr self,
context: EvaluateContext<'_, 'val, T>,
locals: &BTreeMap<&'expr str, Cow<'val, Value>>,
) -> Result<Cow<'val, Value>, Error>where
'expr: 'val,
pub fn apply_with_locals<'expr, 'val, T>(
&'expr self,
context: EvaluateContext<'_, 'val, T>,
locals: &BTreeMap<&'expr str, Cow<'val, Value>>,
) -> Result<Cow<'val, Value>, Error>where
'expr: 'val,
Run the expression to completion and return the result
Locals can be used to augment or override the provided JSON values.