Engine

Struct Engine 

pub struct Engine<T> { /* private fields */ }
Global context for how to evaluate an expression

Mostly this holds all of the functions that can be called during evaluation.

This can typically be created once and used any time an expression evaluation needs to happen.

Implementations

Source

impl<T> Engine<T>

pub fn empty() -> Self

Create a new engine that does not contain any defined functions

pub fn function(&self, name: &str) -> Option<&dyn Function<T>>

Get a function by name

pub fn set_function( &mut self, name: impl ToString, function: Box<dyn Function<T>>, )

Add or override a function with the given name

use rets_expression::{Engine, function::{ListFunction, SetFunction}};

let mut engine = Engine::empty();
engine.set_function("LIST", Box::new(ListFunction));
engine.set_function("SET", Box::new(SetFunction));
pub fn with_function( self, name: impl ToString, function: Box<dyn Function<T>>, ) -> Self

Add or override a function with the given name

use rets_expression::{Engine, function::{ListFunction, SetFunction}};

let engine = Engine::empty()
    .with_function("LIST", Box::new(ListFunction))
    .with_function("SET", Box::new(SetFunction));