EvaluateContext

Struct EvaluateContext 

pub struct EvaluateContext<'engine, 'json, T> { /* private fields */ }
The information required to evaluate an expression

Implementations

Source

impl<'engine, 'json> EvaluateContext<'engine, 'json, ()>

pub fn new(engine: &'engine Engine<()>, value: &'json Value) -> Self

Create a new evaluation context with the provided value

Source

impl<'engine, 'json, T> EvaluateContext<'engine, 'json, T>

pub fn new_with_state( engine: &'engine Engine<T>, value: &'json Value, state: T, ) -> Self

Create a new evaluation context with the provided value and data

pub fn with_previous(self, previous_value: &'json Value) -> Self

Add the previous data to use when evaluating

Previous data is what gets used when an expression uses LAST FieldName.

pub fn set_previous(self, previous_value: Option<&'json Value>) -> Self

Set the previous data to use when evaluating

Previous data is what gets used when an expression uses LAST FieldName.

pub fn value(&self) -> &'json Value

Get a reference to the value within this context

pub fn previous_value(&self) -> Option<&'json Value>

Get a reference to the previous value within this context, if available

pub fn engine(&self) -> &'engine Engine<T>

Get a reference to the engine used when creating this context

pub fn state(&self) -> &T

Get a reference to the state used when creating this context

pub fn state_mut(&mut self) -> &mut T

Get a mutable reference to the state used when creating this context