Struct rets_expression::LastFieldNode
source · #[non_exhaustive]pub struct LastFieldNode {
pub name: String,
/* private fields */
}Expand description
A node representing a reference to a field from the previous value
E.g. LAST MlsStatus
Fields (Non-exhaustive)
This struct is marked as non-exhaustive
Non-exhaustive structs could have additional fields added in future. Therefore, non-exhaustive structs cannot be constructed in external crates using the traditional
Struct { .. } syntax; cannot be matched against without a wildcard ..; and struct update syntax will not work.name: StringThe name of the field that is being referenced
Implementations