Enum rets_expression::ExpressionOp

source ·
pub enum ExpressionOp {
Show 14 variants Add, Sub, Mul, Div, Mod, Concat, Lt, Lte, Gt, Gte, Eq, Ne, Contains, In,
}
A binary operation in an OpNode

Variants



Add

Add two expressions A + B



Sub

Subtract one expression from another A - B



Mul

Multiple two expressions A * B



Div

Divide one expression by another A / B



Mod

Take the modulus of one number with another A .MOD. B



Concat

Concatenate two strings A || B



Lt

Compare two expressions using a less-than comparison A < B



Lte

Compare two expressions using a less-than-or-equal-to comparison A <= B



Gt

Compare two expressions using a greater-than comparison A > B



Gte

Compare two expressions using a greater-than-or-equal-to comparison A >= B



Eq

Compare two expressions using an equal-to comparison A = B



Ne

Compare two expressions using a not-equal-to comparison A != B



Contains

Determine if one expression contains another expression A .CONTAINS. B



In

Determine if one expression is in another expression A .IN. B

Implementations

source

impl ExpressionOp

source

pub fn apply(self, left: &Value, right: &Value) -> Result<Value, Error>

Apply the operation to two values